#!/usr/bin/env python3

import os
import sys
import time
import numpy as np
import simpleaudio as sa


class Player:
    def __init__(self, volume: float = 0.3,
                mute_output: bool = False):

        if volume < 0 or volume > 1:
            raise ValueError("Volume must be a float between 0 and 1")

        # Frequencies for the lowest octave
        self.note_frequencies =  {
            'A': 27.50000,
            'B': 30.86771,
            'C': 16.35160,
            'D': 18.35405,
            'E': 20.60172,
            'F': 21.82676,
            'G': 24.49971
        }

        self.volume = volume
        self.mute_output = mute_output
        self.rate = 44100
        self.freq = 0
        self.fade = 800
        self._valid_note = True
        self._fade_in = np.arange(0., 1., 1 / self.fade)
        self._fade_out = np.arange(1., 0., -1 / self.fade)
        self._play_obj = None
        self._destructor_sleep = 0

    def __set_base_frequency(self, note: str):
        letter = note[:1].upper()
        try:
            self.freq = self.note_frequencies[letter]
        except:
            self._valid_note = False
            print("Error: invalid note: '"
                                    + note[:1]
                                    + "'",
                                    file=sys.stderr)

    def __set_octave(self, octave: str = '4'):
        if not self._valid_note:
            return
        try:
            octaveValue = int(octave)
            if octaveValue < 0 or octaveValue > 8:
                raise ValueError('octave value error')
            self.freq *= (2 ** octaveValue)
        except:
            self._valid_note = False
            print("Error: invalid octave: '"
                                    + octave
                                    + "'",
                                    file=sys.stderr)

    def __set_semitone(self, symbol: str):
        if not self._valid_note:
            return
        if symbol == '#':
            self.freq *= (2 ** (1. / 12.))
        elif symbol == 'b':
            self.freq /= (2 ** (1. / 12.))
        else:
            self._valid_note = False
            print("Error: invalid symbol: '"
                                    + symbol
                                    + "'",
                                    file=sys.stderr)

    def __calc_frequency(self, note: str):
        self.__set_base_frequency(note)
        if len(note) == 1:
            self.__set_octave()
        elif len(note) == 2:
            if note[1:2] == '#' or note[1:2] == 'b':
                self.__set_octave()
                self.__set_semitone(note[1:2])
            else:
                self.__set_octave(note[1:2])
        elif len(note) == 3:
            self.__set_octave(note[1:2])
            self.__set_semitone(note[2:3])
        else:
            if self._valid_note:
                print("Errror: invalid note: '"
                                        + note
                                        + "'",
                                        file=sys.stderr)
                self._valid_note = False

    def __wait_for_prev_sound(self):
        if self._play_obj is not None:
            while self._play_obj.is_playing(): pass


    def __write_stream(self, duration: float):
        t = np.linspace(0, duration, int(duration * self.rate), False)
        audio = np.sin(self.freq * t * 2 * np.pi)
        audio *= 32767 / np.max(np.abs(audio))
        audio *= self.volume

        if len(audio) > self.fade:
            audio[:self.fade] *= self._fade_in
            audio[-self.fade:] *= self._fade_out

        audio = audio.astype(np.int16)

        self.__wait_for_prev_sound()
        self._play_obj = sa.play_buffer(audio, 1, 2, self.rate)

    def __print_played_note(self, note: str, duration: float):
        if self.mute_output or not self._valid_note:
            return
        if note == 'pause':
            print("Pausing for " + str(duration) + "s")
        else:
            print("Playing " + note + " (" + format(self.freq, '.2f') + " Hz) for " + str(duration) + "s")

    def play_note(self, note: str, duration: float = 0.5):
        self._valid_note = True
        if note == 'pause':
            self.__wait_for_prev_sound()
            self.__print_played_note(note, duration)
            time.sleep(duration)
            self._destructor_sleep = 0
        else:
            self.__calc_frequency(note)
            if self._valid_note:
                self.__write_stream(duration)
                self.__print_played_note(note, duration)
                self._destructor_sleep = duration

    def __del__(self):
        time.sleep(self._destructor_sleep)


import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

README = (HERE / "PYPI_README.md").read_text()

setup(
    name="musicalbeeps",
    version="0.2.9",
    description="Play sound beeps corresponding to musical notes.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/MaelDrapier/musicalbeeps",
    author="Maël Drapier",
    author_email="mael.drapier@gmail.com",
    license="MIT",
    keywords="music musical note notes beep beeps play player sound frequency",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Multimedia :: Sound/Audio",
        "Topic :: Multimedia :: Sound/Audio :: Players"
    ],
    packages=find_packages(),
    install_requires=["numpy", "simpleaudio"],
    entry_points={
        "console_scripts": [
            "musicalbeeps=musicalbeeps.script:main",
        ]
    },
)


#!/usr/bin/env python3

import os
import sys
import argparse
import musicalbeeps


def setup_argparse():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                    description='Play sound beeps corresponding to musical notes.',
                                    epilog='''\

how to play notes:
    Notes are read from a file passed as argument, or directly from the
    standard input. Each note must be on a new line.

note format:
    Each note must be formatted like so: 'A5#:1.5' (without quotes)
    Where:
        - 'A' is the note (between A and G, can be lowercase)
        - '5' is the octave (between 0 and 8, default=4)
        - '#' (or 'b') is optional and used to play a sharp or flat note
        - ':1.5' is the duration of the note (1.5 seconds here, default=0.5)

pause:
    You can pause the player by replacing the note by the 'pause' word.
    For exemple, 'pause:5' will pause the player for 5 seconds.''')
    parser.add_argument("file",
                        nargs="?",
                        help="a file containing musical notes")
    parser.add_argument("--silent",
                        help="disable player output",
                        action='store_true')
    parser.add_argument("--volume",
                        help="volume between 0 and 1 (default=0.3)",
                        type=float,
                        default=0.3)
    args = parser.parse_args()
    if args.file:
        input_file = open(args.file, 'r')
    else:
        input_file = sys.stdin
    return args, input_file

def player_loop(args, input_file):
    notes_player = musicalbeeps.Player(args.volume, args.silent)

    for line in input_file:
        valid_duration = True
        line = line.rstrip()
        if len(line) > 0:
            try:
                note, duration_str = line.split(':')
            except:
                note, duration_str = line, '.5'
            try:
                duration = float(duration_str)
            except:
                valid_duration = False
                print("Error: invalid duration: '"
                                        + duration_str
                                        + "'",
                                        file=sys.stderr)
            if valid_duration:
                notes_player.play_note(note, duration)

def main():
    try:
        args, input_file = setup_argparse()
        player_loop(args, input_file)
        if not args.silent:
            print("Done")
        if input_file is not sys.stdin:
            input_file.close
    except KeyboardInterrupt:
        pass
    except Exception as e:
        print(e, file=sys.stderr)

if __name__ == "__main__":
    main()